@@ -11,7 +11,7 @@ Pandoc - interface to the Pandoc document converter
 
 =cut
 
-use version 0.77; our $VERSION = version->declare('0.3.1');
+our $VERSION = '0.4.0';
 
 use Carp 'croak';
 use File::Which;
@@ -19,9 +19,14 @@ use IPC::Run3;
 use parent 'Exporter';
 our @EXPORT = qw(pandoc);
 
+use Pandoc::Version;
+$VERSION = Pandoc::Version->new($VERSION);
+
 our $PANDOC;
 our $PANDOC_VERSION_SPEC = qr/^(\d+(\.\d+)*)$/;
 
+our $PANDOC_PATH ||= $ENV{PANDOC_PATH} || 'pandoc';
+
 sub import {
     shift;
 
@@ -44,7 +49,7 @@ sub VERSION {
 sub new {
     my $pandoc = bless { }, shift;
 
-    my $bin = (@_ and $_[0] !~ /^-./) ? shift : 'pandoc';
+    my $bin = (@_ and $_[0] !~ /^-./) ? shift : $PANDOC_PATH;
     $pandoc->{bin} = which($bin);
 
     $pandoc->{arguments} = [];
@@ -59,9 +64,21 @@ sub new {
     croak "pandoc executable not found\n" unless
         $out and $out =~ /^pandoc (\d+(\.\d+)+)/;
 
-    $pandoc->{version} = version->parse("v$1");
+    $pandoc->{version} = Pandoc::Version->new($1);
     $pandoc->{data_dir} = $1 if $out =~ /^Default user data directory: (.+)$/m;
 
+	my $LIBRARY_NAME = qr/\pL\w*(?:-\pL\w*)*/;
+	my $LIBRARY_VERSION = qr/\s+($LIBRARY_NAME)\s+(\d+(?:\.\d+)*),?/;
+
+	my %libs;
+	if ( $out =~ /^Compiled with($LIBRARY_VERSION+)/m ) {
+        %libs = $1 =~ /$LIBRARY_VERSION/g;
+        for my $name ( keys %libs ) {
+            $libs{$name} = Pandoc::Version->new( $libs{$name} );
+        }
+	}
+    $pandoc->{libs} = \%libs;
+
     return $pandoc;
 }
 
@@ -129,8 +146,8 @@ sub convert {
 
     my $utf8 = utf8::is_utf8($in);
 
-    my $status = $pandoc->run( [ '-f' => $from, '-t' => $to, @_ ],
-        in => \$in, out => \$out, err => \$err );
+    my %opts = (in => \$in, out => \$out, err => \$err);
+    my $status = $pandoc->run( [ '-f' => $from, '-t' => $to, @_ ], \%opts );
 
     croak($err || "pandoc failed with exit code $status") if $status;
 
@@ -156,11 +173,25 @@ sub parse {
     Pandoc::Elements::pandoc_json($json);
 }
 
+sub file {
+    my $pandoc = shift;
+    $pandoc->require('1.12.1');
+
+    my ($json, $err);
+    my @args = ('-t' => 'json', @_);
+    my $status = $pandoc->run( \@args, out => \$json, err => \$err );
+    croak($err || "pandoc failed with exit code $status") if $status;
+
+    #utf8::decode($out) if $utf8;
+
+    require Pandoc::Elements;
+    Pandoc::Elements::pandoc_json($json);
+}
+
 sub require {
     my ($pandoc, $version) = @_;
     $pandoc = do { $PANDOC //= Pandoc->new } if $pandoc eq 'Pandoc'; 
     unless ($pandoc->version($version)) {
-        $version = version->parse($version =~ /^v/ ? $version : "v$version");
         croak "pandoc $version required, only found ".$pandoc->{version}."\n"
     }
     return $pandoc;
@@ -171,13 +202,7 @@ sub version {
     return unless $pandoc and $pandoc->{version};
 
     # compare against given version
-    if (@_) {
-        my $v = shift;
-        $v = "v$v" unless $v =~ /^v/; # force dotted decimal format
-        my $version = eval { version->parse($v) } or
-            croak "Invalid version format: $v";
-        return if $version > $pandoc->{version};
-    }
+    return if @_ and $pandoc->{version} < $_[0];
 
     return $pandoc->{version};
 }
@@ -206,32 +231,38 @@ sub arguments {
     @{$pandoc->{arguments}};
 }
 
-sub _help { # not documented. may change to return structured data
-    my ($pandoc) = @_;
-
-	unless (defined $pandoc->{help}) {
-		my $help;
-    	$pandoc->run('--help', { out => \$help });
-		for my $inout (qw(Input Output)) {
-			$help =~ /^$inout formats:\s+([a-z_0-9,\+\s*]+)/m;
-			$pandoc->{lc($inout).'_formats'} = [ split /,\s+|\s+/, $1 ]
-		}
-	    $pandoc->{help} = $help;
+sub _list {
+    my ($pandoc, $which) = @_;
+    if (!$pandoc->{$which}) {
+        if ($pandoc->version('1.18')) {
+            my $list = "";
+            my $command = $which;
+            $command =~ s/_/-/g;
+            $pandoc->run("--list-$command", { out => \$list });
+	        $pandoc->{$which} = [ split /\n/, $list ];
+        } elsif (!defined $pandoc->{help}) {
+            my $help;
+            $pandoc->run('--help', { out => \$help });
+            for my $inout (qw(Input Output)) {
+                $help =~ /^$inout formats:\s+([a-z_0-9,\+\s*]+)/m or next;
+                $pandoc->{lc($inout).'_formats'} = [ split /,\s+|\s+/, $1 ]
+            }
+            $pandoc->{help} = $help;
+        }
     }
-
-	$pandoc->{help} 
+    @{$pandoc->{$which}};
 }
 
 sub input_formats {
-    my ($pandoc) = @_;
-	$pandoc->_help;
-    @{$pandoc->{input_formats}};
+    $_[0]->_list('input_formats');
 }
 
 sub output_formats {
-    my ($pandoc) = @_;
-	$pandoc->_help;
-    @{$pandoc->{output_formats}};
+    $_[0]->_list('output_formats');
+}
+
+sub libs {
+    $_[0]->{libs};
 }
 
 1;
@@ -269,11 +300,13 @@ __END__
   pandoc or die "pandoc executable not found";
 
   # check minimum version
-  pandoc->version(1.12) or die "pandoc >= 1.12 required";
+  pandoc->version > 1.12 or die "pandoc >= 1.12 required";
 
   # access properties
   say pandoc->bin." ".pandoc->version;
   say "Default user data directory: ".pandoc->data_dir;
+  say "Compiled with: ".join(", ", keys %{ pandoc->libs });
+  say pandoc->libs->{'highlighting-kate'};
 
   # create a new instance with default arguments
   my $md2latex = Pandoc->new(qw(-f markdown -t latex --smart));
@@ -288,8 +321,9 @@ __END__
   # utility method to convert from string
   $latex = pandoc->convert( 'markdown' => 'latex', '*hello*' );
 
-  # utility method to parse abstract syntax tree
+  # utility methods to parse abstract syntax tree (requires Pandoc::Elements)
   $doc = pandoc->parse( markdown => '*hello* **world!**' );
+  $doc = pandoc->file( 'example.md' );
 
 =head1 DESCRIPTION
 
@@ -309,9 +343,10 @@ used by function pandoc.
 
 =head2 pandoc
 
-If called without parameters, this function returns a global instance of
-class Pandoc to execute L<methods|/METHODS>, or C<undef> if no pandoc
-executable was found. 
+If called without parameters, this function returns a global instance of class
+Pandoc to execute L<methods|/METHODS>, or C<undef> if no pandoc executable was
+found. The location and/or name of pandoc executable can be set with
+environment variable C<PANDOC_PATH> (set to "pandoc" by default).
 
 =head2 pandoc( ... ) 
 
@@ -397,6 +432,15 @@ Parse a string in format C<$from> to a L<Pandoc::Document> object. Additional
 pandoc options such as C<--smart> and C<--normalize> can be passed. This method
 requires at least pandoc version 1.12.1 and the Perl module L<Pandoc::Elements>.
 
+=head2 file( $filename [, @arguments ] )
+
+Parse from a file to a L<Pandoc::Document> object. Additional pandoc options
+can be passed, for instance to set input format (markdown by default):
+
+  pandoc->file('example.html', '-f', 'html')
+
+Requires at least pandoc version 1.12.1 and the Perl module L<Pandoc::Elements>.
+
 =head2 require( $minimum_version )
 
 Return the Pandoc instance if its version number is at least as high as the
@@ -407,11 +451,24 @@ executable was found.
 
 =head2 version( [ $minimum_version ] )
 
-Return the pandoc version as L<version> object. The version number is
-normalized to always start with a small letter "v", for instance "v1.16.0.2" or
-"v1.17".  If a minimum version is passed, the method returns undef if the
-pandoc version is lower. To check whether pandoc is available with a given
-minimal version use one of:
+Return the pandoc version as L<Pandoc::Version> object.  If a minimum version
+is given, the method returns undef if the pandoc version is lower. To check
+whether pandoc is available with a given minimal version use one of:
+
+
+=head2 require( $minimum_version )
+
+Return the Pandoc instance if its version number is at least as high as the
+given minimum version. Throw an error otherwise.  This method can also be
+called as constructor: C<< Pandoc->require(...) >> is equivalent to C<<
+pandoc->require >> but throws a more meaningful error message if no pandoc
+executable was found.
+
+=head2 version( [ $minimum_version ] )
+
+Return the pandoc version as L<Pandoc::Version> object.  If a minimum version
+is given, the method returns undef if the pandoc version is lower. To check
+whether pandoc is available with a given minimal version use one of:
 
   Pandoc->require( $minimum_version)                # true or die
   pandoc and pandoc->version( $minimum_version )    # true or false
@@ -437,12 +494,20 @@ Return a list of supported input formats.
 
 Return a list of supported output formats.
 
+=head2 libs
+
+Return a hash of Haskell libraries compiled into pandoc executable and their
+version numbers as L<Pandoc::Version> objects.
+
 =head1 SEE ALSO
 
 Use L<Pandoc::Elements> for more elaborate document processing based on Pandoc.
 Other Pandoc related but outdated modules at CPAN include
 L<Orze::Sources::Pandoc> and L<App::PDoc>.
 
+See L<pyandoc|https://pypi.python.org/pypi/pyandoc/> for a similar Pandoc
+wrapper in Python.
+
 =head1 AUTHOR
 
 Jakob Voß
